/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  SPI.c - Definition file for utility functions
 *  drv81xxx-Q1 SPI FIRMWARE frame work
 *  01/05/2024
 *  Murugavel Raju
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "drv81xxx-Q1/include/drv81xxx-Q1.h"
#include "SPI.h"
/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/

// SPI Initialization
void spi_init(void)
{
    P8SEL |= SDO | SDI | SCLK;                                            // SPI pins configurations

    UCB1CTL1 |= UCSWRST;                                                  // SPI reset
    UCB1CTL0 = UCMSB | UCMST | UCSYNC;                                    // SPI controller MSB first
    UCB1CTL1 |= UCSSEL_2;                                                 // SMCLK
    UCB1BR0 |= 10;                                                        // 2MHz
    UCB1BR1 = 0;                                                          //
    UCB1CTL1 &= ~UCSWRST;                                                 // Initialize USCI state machine
}

// This SPI function is used to read the device registers
uint16_t spi_readRegister(uint8_t address)
{
    volatile uint8_t i = 0;
    volatile uint16_t reg_value;
    volatile uint8_t dataMSB;
    volatile uint8_t dataLSB;

    while (i < 2)                                                         // Read the same register twice to get the data
    {
        reg_value = 0;                                                    // Clear reg_value for new read
        reg_value |= ((address << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK);   // Configure register address value
        if (address == 0x3F)
            reg_value |= SPI_STDDIAG_RD;                                  // SPI Standard Diagnosis register read mask
        else
            reg_value |= SPI_READ_MASK;                                   // SPI register read mask

        set_nSCS_lo;                                                      // nSCS low

        while (!(UCB1IFG & UCTXIFG));                                     // USCI_B1 TX buffer ready?
        UCB1TXBUF = (uint8_t)((reg_value >>8) & 0xFF);                    // Transmit the Address(MSB Byte)
        while(UCB1STAT & UCBUSY);                                         // Wait till Transmission is complete
        //while(UCB1IFG & UCRXIFG);                                         // Wait till first byte received
        dataMSB = UCB1RXBUF;                                              // Save the received byte, MSB byte
        UCB1TXBUF = (uint8_t)(reg_value & 0xFF);                          // Transmit Second byte
        while(UCB1STAT & UCBUSY);                                         // Wait till Transmission is complete
        dataLSB = UCB1RXBUF;                                              // Save the received byte, LSB byte

        set_nSCS_hi;                                                      // nSCS high
        reg_value = ((dataMSB<<8) | dataLSB);                             // 16-bit data
        i++;
    }

     return (reg_value);
}

// This SPI function is used to write to the device registers
uint16_t spi_writeRegister(uint8_t address, uint16_t data)
{
  volatile uint16_t reg_value = 0;
  volatile uint8_t dataMSB;
  volatile uint8_t dataLSB;

  reg_value |= (((address << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK) | SPI_WRITE_MASK); // Setting up register address and write mask values
  if (address == PWM0_Configuration_Register || address == PWM1_Configuration_Register) // Check if 10-bit data registers
  {
      reg_value |= ((data << SPI_DATA_POS) & SPI_10BIT_DATA_MASK);        // Adding 10-bit data value
  }
  else
      reg_value |= ((data << SPI_DATA_POS) & SPI_DATA_MASK);              // Adding 8-bit data value

  set_nSCS_lo;                                                            // nSCS low
  while (!(UCB1IFG & UCTXIFG));                                           // USCI_B1 TX buffer ready?
  UCB1TXBUF = (uint8_t)((reg_value>>8) & 0xFF);                           // Transmit first Byte,MSB-Byte
  while(UCB1IFG & UCRXIFG);                                               // Wait till first byte received
  dataMSB = UCB1RXBUF;                                                    // Save the received byte, MSB byte
  while (!(UCB1IFG & UCTXIFG));                                           // USCI_B1 TX buffer ready
  UCB1TXBUF = (uint8_t)(reg_value & 0xFF);                                // Transmit Second Byte, LSB-Byte
  while(UCB1STAT & UCBUSY);                                               // Wait till Transmission is complete
  dataLSB = UCB1RXBUF;                                                    // Save the received byte, LSB byte
  set_nSCS_hi;                                                            // nSCS high

  reg_value = ((dataMSB<<8) | dataLSB);                                   // 16-bit data (this is the data output from previous response - unused in this version)

  // Do a manual read to update the memory with the newly written value
  gBrushedObj.ManReadAddr = address;
  gBrushedObj.ManReadCmd = 1;

  return 0;
}
