/* --COPYRIG  HT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  drv81xxx-Q1.c - Definition file for utility functions
 *  DRV81xxx-Q1EVM_FIRMWARE
 *  01/05/2024
 *  Murugavel Raju
 *
 *******************************************************************************/

/********************************************************************************
 * FIRMWARE VERSION INFORMATION
 *******************************************************************************/
#define FIRMWARE_VERSION 0.12                 // Updated functions

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "drv81xxx-Q1/include/drv81xxx-Q1.h"
#include "SPI.h"
#include "uart.h"

/********************************************************************************
 * Data Structures exported for GUI Composer
 *******************************************************************************/
#pragma DATA_SECTION(gBrushedObj,".drvVarSection");
#pragma DATA_SECTION(gDeviceSpiReg,".drvRegSection");

volatile DRV8xxxQ1_REG_t gDeviceSpiReg;
volatile DRV8xxxQ1_Obj_t gBrushedObj;

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
// Clock Initialization
static inline void clock_Init(void)
{
    PMMCTL0 = PMMPW | PMMCOREV_3;             // PMM Core Voltage 1.85 V for 20 MHz CPU operation
    __delay_cycles(100000);                   // Settling delay for the core regulator

    UCSCTL3 = SELREF_2;                       // Set DCO FLL reference = REFO
    UCSCTL4 |= SELA_2 + SELS_3;               // Set ACLK = REFO and SMCLK = DCO
    UCSCTL1 = DCORSEL_6;                      // Select DCO range 20MHz operation
    UCSCTL2 = 609;                            // FLL multiplier
    __delay_cycles(625000);                   // Settling delay for DCO, 32 x 32 x 20 MHz / 32768 Hz = 625000 = MCLK cycles
}

// GPIO Initialization. This block configures all the DRV8434S PINs.
static inline void gpio_Init(void)
{
    // Port P1 configuration
    P1OUT = 0;                                                       // All outputs low
    P1DIR = 0;                                                       // Default inputs
    P1DIR = MCU_STATUS;                                              // MCU_STATUS output for led

    // Port P2 configuration
    P2OUT = 0;                                                       // All outputs low
    P2DIR = 0;                                                       // Default inputs
    P2DIR = FAULT_STATUS | TP3 | TP2 | TP1;                          // FAULT_STATUS output for led and TP!, 2 & 3 output

    // Port P3 configuration
    P3OUT = 0;                                                       // All outputs low
    P3DIR = 0;                                                       // Default inputs
    P3DIR = GUI_STATUS | DRV_STATUS | TP6;                           // GUI_STATUS, DRV_STATUS output for led and TP6 output

    // Port P4 configuration
    P4OUT = 0;                                                       // All outputs low
    P4DIR = 0;                                                       // Default inputs
    P4DIR = CH8 | CH7 | CH6  | CH5 | CH2 | CH1;                      // Channels 1,2,5,6,7,8 led outputs

    // Port P5 configuration
    P5OUT = 0;                                                       // All outputs low
    P5DIR = 0;                                                       // Default inputs
    P5DIR = TP5 | TP4;                                               // IDR_0, IDR_1, IDR_2, VREF+ and VREF- are inputs, TP4 and 6 outputs
    P5SEL = BIT1 | BIT0;                                             // P5.0 = VeREF+ and P5.1 = VeREF-

    // Port P6 configuration
    P6OUT = 0;                                                       // All outputs low
    P6DIR = 0xFE;                                                    // Unused pins as outputs
    P6SEL = VBAT_MON;                                                // VBAT_MON A0 analog input

    // Port P7 configuration
    P7OUT = 0;                                                       // All outputs low
    P7DIR = 0xFF;                                                    // Unused pins as outputs

    // Port P8 configuration
    P8OUT = nSCS;                                                    // All outputs except nSCS low
    P8DIR = 0;                                                       // Default inputs
    P8DIR = nSCS | CH4 | CH3;                                        // All except nSCS, channels 3 and 4 led as inputs

    // Port P9 configuration
    P9OUT = 0;                                                       // All outputs low
    P9DIR = 0;                                                       // Default inputs
    P9DIR = nSLEEP | IN_1 | IN_0;                                    // nSLEEP, IN_1 and IN_0 signal outputs
}

// Watchdog initialization
static inline void watchdog_Init(void)
{
    WDTCTL = WDT_ARST_250;                                           // Watchdog mode with 250ms reset time using ACLK
}

// Setup Timer_A2 for periodic wake-up from LPM0 and to Blink the STATUS LED
static inline void ta2_init(void)
{
    // select SMCLK, divide by 8, stop
    TA2CTL = TASSEL_2 | ID_3 | MC_0 | TACLR;
    TA2CCR0 = 20001;                                                 // 8 ms interrupt SMCLK = 20 MHz, TACLK = 2.5 MHz
    TA2CCTL0 |= CCIE;                                                // TIMER_A3CCR0 dedicated interrupt
    TA2CTL |= MC_1;                                                  // Start Timer_A2 in up mode
}

// Read EVMID information of the EVM hardware
void getEVMID(void)
{
    /* Get identify of the EVM connected
     *
     * DRV81xxx-Q1 EVMs have on board ID resistors inputs on
     * the MCU GPIO. ID0 to ID4 inputs uniquely identify the EVM connected
     * as well as whether it has pre-production or production silicon.
     *
     *   IDx=1       IDx=0      Pin state
     *     H                    Pull-up installed
     *                  L       Pull-down installed
     *
     *   P5.5 = ID2
     *   P5.4 = ID1
     *   P5.3 = ID0
     *   P1.0 = ID3
     *   P1.1 = ID4
     *
     *               EVM HW     ID3 ID2 ID1 ID0           ID4     EVM OPN
     * DRV81008-Q1   MD075       L   L   L   L   0x0       X    DRV81008-Q1EVM
     * DRV81004-Q1               L   L   L   H   0x1       X    DRV81004-Q1EVM
     * DRV81080-Q1               L   L   H   L   0x2       X    DRV81080-Q1EVM
     * DRV81242-Q1               L   L   H   H   0x3       X    DRV81242-Q1EVM
     * DRV81602-Q1               L   H   L   L   0x4       X    DRV81602-Q1EVM
     * DRV81620-Q1               L   H   L   H   0x5       X    DRV81620-Q1EVM
     *
     * ID4 = L, pre-production silicon and ID5 = H, production silicon
     */

    uint8_t evmID = 0;

    evmID = ((read_ID2_1_0_pins) >> 3);
    gBrushedObj.deviceID = evmID;

    gBrushedObj.deviceID = 0;          // ******* For testing DRV81008 E1 EVM

    // ID4
    if (read_ID4_pin)                  // ID4 pull-up
        gBrushedObj.revID = REV_PG2_0;
    else                               // ID4 pull-down
        gBrushedObj.revID = REV_PG1_0;
}

/*******************************************************************************
 * API IMPLEMENTATION
 *******************************************************************************/
// Initialize common GUI variables and stepper drivers GUI variables
void drv81xxxQ1_init()
{
    gBrushedObj.Firmware_Version = FIRMWARE_VERSION;
    gBrushedObj.nSleep = 1;
    gBrushedObj.IN0 = 0;
    gBrushedObj.IN1 = 0;
    gBrushedObj.revID = 0;
    gBrushedObj.AutoRead = 1;
}

// Routine for updating registers from the GUI
void UpdateGUI_Register()
{
    uint16_t drvDataNew;
    uint8_t drvRegAddr;

    if (gBrushedObj.nSleep)
    {
       if (gBrushedObj.ManWriteCmd)
       {
           // Register Manual Write
           drvRegAddr = gBrushedObj.ManWriteAddr;
           drvDataNew = gBrushedObj.ManWriteData;

           spi_writeRegister((uint8_t) drvRegAddr, (uint16_t) drvDataNew);

           switch (drvRegAddr)
           {
             // Do not Write to Read only Registers
             case Standard_Diagnosis_Register:
                 // Read only gDeviceSpiReg.standard_diag_reg = drvDataNew;
                 break;
             case Output_Control_Register:
                 gDeviceSpiReg.output_control_reg = drvDataNew;
                 break;
             case Bulb_Inrush_Register:
                 if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                     gDeviceSpiReg.bulb_inrush_reg = drvDataNew;
                 break;
             case IN0_Map_Register:
                 gDeviceSpiReg.IN0_map_reg = drvDataNew;
                 break;
             case IN1_Map_Register:
                 gDeviceSpiReg.IN1_map_reg = drvDataNew;
                 break;
             case Input_Status_Monitor_Register:
                 // Read only gDeviceSpiReg.input_status_monitor_reg = drvDataNew;
                 break;
             case Open_Load_Current_Control_Register:
                 gDeviceSpiReg.OL_current_control_reg = drvDataNew;
                 break;
             case Output_Status_Monitor_Register:
                 // Read only gDeviceSpiReg.output_status_monitor_reg = drvDataNew;
                 break;
             case Open_Load_at_ON_Register:
                 // Read only gDeviceSpiReg.OL_at_ON_reg = drvDataNew;
                 break;
             case EN_OLON_Register:
                 if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                     gDeviceSpiReg.EN_OLON_reg = drvDataNew;
                 break;
             case Configuration_Register:
                 gDeviceSpiReg.config_reg = drvDataNew;
                 break;
             case Output_Clear_Latch_Register:
                 gDeviceSpiReg.output_clear_latch_reg = drvDataNew;
                 break;
             case FPWM_Register:
                 if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                     gDeviceSpiReg.FPWM_reg = drvDataNew;
                 break;
             case PWM0_Configuration_Register:
                 if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                     gDeviceSpiReg.PWM0_config_reg = drvDataNew;
                 break;
             case PWM1_Configuration_Register:
                 if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                     gDeviceSpiReg.PWM1_config_reg = drvDataNew;
                 break;
             case PWM_OUT_Register:
                 if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                     gDeviceSpiReg.PWM_out_reg = drvDataNew;
                 break;
             case MAP_PWM_Register:
                 if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                     gDeviceSpiReg.MAP_PWM_reg = drvDataNew;
                 break;
             case Configuration2_Register:
                 gDeviceSpiReg.Config_reg2 = drvDataNew;
                 break;
             default:
                 break;
           } // End of Switch
           gBrushedObj.ManWriteCmd = false;
       } // End of Manual Write Command

       // Read Register Values
       if (ReadCmd)
       {
           gDeviceSpiReg.standard_diag_reg = spi_readRegister(Standard_Diagnosis_Register);
           gDeviceSpiReg.output_control_reg = spi_readRegister(Output_Control_Register);
           if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
               gDeviceSpiReg.bulb_inrush_reg = spi_readRegister(Bulb_Inrush_Register);
           gDeviceSpiReg.IN0_map_reg = spi_readRegister(IN0_Map_Register);
           gDeviceSpiReg.IN1_map_reg = spi_readRegister(IN1_Map_Register);
           gDeviceSpiReg.input_status_monitor_reg = spi_readRegister(Input_Status_Monitor_Register);
           gDeviceSpiReg.OL_current_control_reg = spi_readRegister(Open_Load_Current_Control_Register);
           gDeviceSpiReg.output_status_monitor_reg = spi_readRegister(Output_Status_Monitor_Register);
           if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
               gDeviceSpiReg.OL_at_ON_reg = spi_readRegister(Open_Load_at_ON_Register);
           if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
               gDeviceSpiReg.EN_OLON_reg = spi_readRegister(EN_OLON_Register);
           gDeviceSpiReg.config_reg = spi_readRegister(Configuration_Register);
           gDeviceSpiReg.output_clear_latch_reg = spi_readRegister(Output_Clear_Latch_Register);
           if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
           {
               gDeviceSpiReg.FPWM_reg        = spi_readRegister(FPWM_Register);
               gDeviceSpiReg.PWM0_config_reg = spi_readRegister(PWM0_Configuration_Register);
               gDeviceSpiReg.PWM1_config_reg = spi_readRegister(PWM1_Configuration_Register);
               gDeviceSpiReg.PWM_out_reg = spi_readRegister(PWM_OUT_Register);
               gDeviceSpiReg.MAP_PWM_reg = spi_readRegister(MAP_PWM_Register);
           }
           gDeviceSpiReg.Config_reg2     = spi_readRegister(Configuration2_Register);

           ReadCmd = false;
       }// End of Registers Read Command

       if (gBrushedObj.ManReadCmd)
       {
           // Register Manual Read
           drvRegAddr = gBrushedObj.ManReadAddr;
           drvDataNew = spi_readRegister((uint8_t) drvRegAddr);

           gBrushedObj.ManReadCmd = false;
           gBrushedObj.ManReadData = drvDataNew;
           switch (drvRegAddr)
           {
           case Standard_Diagnosis_Register:
               gDeviceSpiReg.standard_diag_reg = drvDataNew;
               break;
           case Output_Control_Register:
               gDeviceSpiReg.output_control_reg = drvDataNew;
               break;
           case Bulb_Inrush_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.bulb_inrush_reg = drvDataNew;
               break;
           case IN0_Map_Register:
               gDeviceSpiReg.IN0_map_reg = drvDataNew;
               break;
           case IN1_Map_Register:
               gDeviceSpiReg.IN1_map_reg = drvDataNew;
               break;
           case Input_Status_Monitor_Register:
               gDeviceSpiReg.input_status_monitor_reg = drvDataNew;
               break;
           case Open_Load_Current_Control_Register:
               gDeviceSpiReg.OL_current_control_reg = drvDataNew;
               break;
           case Output_Status_Monitor_Register:
               gDeviceSpiReg.output_status_monitor_reg = drvDataNew;
               break;
           case Open_Load_at_ON_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.OL_at_ON_reg = drvDataNew;
               break;
           case EN_OLON_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.EN_OLON_reg = drvDataNew;
               break;
           case Configuration_Register:
               gDeviceSpiReg.config_reg = drvDataNew;
               break;
           case Output_Clear_Latch_Register:
               gDeviceSpiReg.output_clear_latch_reg = drvDataNew;
               break;
           case FPWM_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.FPWM_reg = drvDataNew;
               break;
           case PWM0_Configuration_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.PWM0_config_reg = drvDataNew;
               break;
           case PWM1_Configuration_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.PWM1_config_reg = drvDataNew;
               break;
           case PWM_OUT_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.PWM_out_reg = drvDataNew;
               break;
           case MAP_PWM_Register:
               if ((gBrushedObj.deviceID) > 1)     // Not present in DRV81008 and DRV81004
                   gDeviceSpiReg.MAP_PWM_reg = drvDataNew;
               break;
           case Configuration2_Register:
               gDeviceSpiReg.Config_reg2 = drvDataNew;
               break;
           default:
               break;
           } // End of Switch
       } // End of Manual Read Command

       gBrushedObj.ManReadCmd = false;
    }
} // End of UpdateGUI_Register()

// Read the GUI settings and update device control
void UpdateDeviceControls_drv81xxxQ1()
{
    // toggle MCU_STATUS LED if count is reached
    if (LED_TOGGLE_CNT > LED_TIMEOUT)
    {
        toggle_status_led;
        LED_TOGGLE_CNT = 0;

        // Measure VM and update to GUI display
        ADC12CTL0 |= ADC12ON;                                        // Enable ADC12
        ADC12IE = 0;                                                 // Disable ADC interrupts
        ADC12CTL1 = ADC12SHP;                                        // configure for single conversion
        ADC12MCTL0 = ADC12INCH_0 + ADC12SREF_6;                      // A0 for VM measurement using MCU 3V3 as reference
        ADC12CTL0 |= ADC12SC + ADC12ENC;                             // Enable and start conversion
        while (ADC12CTL1 & ADC12BUSY);                               // Wait for conversion to complete
        gBrushedObj.VMvoltage = ADC12MEM0;                           // Conversion result stored in ADC12MEM0 sent to GUI display
        if (gBrushedObj.VMvoltage < ADC12scale)
            gBrushedObj.VMvoltage = 0;                               // VM is 0
        ADC12CTL0 = 0;                                               // Disable ADC12
        // End measure VM and update to GUI display
    }

    // nSLEEP control
    if (gBrushedObj.nSleep && !(read_nSLEEP_pin))
    {
        set_nSLEEP_hi;
        _delay_cycles(10000);                                        // 0.5 ms wake up delay
    }
    else if (!gBrushedObj.nSleep && (read_nSLEEP_pin))
    {
        set_nSLEEP_lo;
    }

    // IN0 control
    if (gBrushedObj.IN0 && !(read_IN0_pin))
    {
        set_IN0_hi;
    }
    else if (!gBrushedObj.IN0 && (read_IN0_pin))
    {
        set_IN0_lo;
    }

    // IN1 control
    if (gBrushedObj.IN1 && !(read_IN1_pin))
    {
        set_IN1_hi;
    }
    else if (!gBrushedObj.IN1 && (read_IN1_pin))
    {
        set_IN1_lo;
    }
}

// Set trigger for reading drv84x2 Registers
void drv81xxxQ1_RegistersRead()
{
    RegReadPeriod++;
    if (RegReadPeriod == REG_READ_PERIOD)
    {
        RegReadPeriod = 0;
        if (gBrushedObj.nSleep)
        {
            if (gBrushedObj.AutoRead)                // Periodic read Registers and update Register memory buffer if Auto read in GUI is not off
            {
                ReadCmd = 1;                         // read device Registers and update values to global variables
            }
        }
    }
}// End Set trigger for reading drv84x2 Registers

// MSP430F5338 MCU initialization
void mcu_init()
{
    WDTCTL = WDTPW | WDTHOLD;                                        // Stop Watchdog Timer on POR and PUC
    clock_Init();
    uartInit();
    gpio_Init();
    spi_init();
    ta2_init();
    // watchdog_Init();
} // End MSP430F5338 MCU initialization
